@@ -2,6 +2,7 @@ import click
 from halo import Halo
 
 from .. import convert_to_xlsx, Services
+from ..convert.commons import get_output_path
 
 
 @click.command(context_settings=dict(max_content_width=200))
@@ -10,10 +11,16 @@ from .. import convert_to_xlsx, Services
 @click.option(
     "-o",
     "--output",
-    help="output path for xlsx file relative from current location (ends in .xlsx, overwrites)",
+    help="output path for xlsx file relative from current location (ends in .xlsx, overwrites previous)",
+)
+@click.option(
+    "-n",
+    "--normalize",
+    is_flag=True,
+    help="Normalize the probabilities to range(min, max) for every column",
 )
 @click.pass_context
-def to_xlsx(ctx, path, workers, output):
+def to_xlsx(ctx, path, workers, output, normalize: bool):
     """ Convert invoice to xlsx """
     # Services available
     spinner = Halo(spinner="dots")
@@ -21,9 +28,10 @@ def to_xlsx(ctx, path, workers, output):
     services = Services(ctx, check_up=True)
     spinner.succeed(f"Found dir: {path}\n ")
 
+    # Get absolute output path
+    output_path = get_output_path(path, output)
+
     # Conversion
-    convert_to_xlsx(
-        path, services, workers, output,
-    )
+    convert_to_xlsx(path, services, workers, output_path, normalize)
 
     spinner.succeed("Converted invoice(s) to xlsx")
