@@ -1,7 +1,9 @@
 import os
+import itertools
+import re
+
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from pathlib import Path
-from copy import copy
 
 import click
 from filetype import guess
@@ -10,15 +12,6 @@ from ..services.requests import extract_invoice, validation, validate_vat
 
 
 def run_requests(workers: int, path: str, services) -> dict:
-    single_fieldnames = {
-        "file_name": (None, None, None),
-        "error_message": (None, None, None),
-    }
-    multi_fieldnames = {
-        "file_name": (None, None, None),
-        "row_number": (None, None, None),
-    }
-    default_cols = set().union(single_fieldnames.keys(), multi_fieldnames.keys())
     files = get_filenames(path)
     result, vat_validation_result = {}, {}
 
@@ -61,18 +54,11 @@ def run_requests(workers: int, path: str, services) -> dict:
                 finally:
                     result[idx]["file_name"] = (file_name, None, None)
 
-                collect_column_names(result[idx], single_fieldnames, multi_fieldnames)
                 bar.update(1)
 
     if not result:
         quit(f"No files found in path")
-    return (
-        result,
-        single_fieldnames,
-        multi_fieldnames,
-        default_cols,
-        vat_validation_result,
-    )
+    return (result, vat_validation_result)
 
 
 def flatten_invoice(invoice, validation):
@@ -141,25 +127,6 @@ def flatten_invoice(invoice, validation):
     return return_dict
 
 
-def collect_column_names(
-    extracted_doc: dict, single_fieldnames: dict, multi_fieldnames: dict
-):
-    """Iterate through extracted invoice and write fieldnames to single or multi
-    cardinality
-
-    Arguments:
-        extracted_invoice {[type]} -- [description]
-        multi_fieldnames {[type]} -- [description]
-        single_fieldnames {[type]} -- [description]
-    """
-    for col_name in extracted_doc.keys():
-        if col_name[-1].isdigit():
-            label = "_".join(col_name.split("_")[:-1])
-            multi_fieldnames[label] = (None, None, None)
-        else:
-            single_fieldnames[col_name] = (None, None, None)
-
-
 def read_pdf(pdf_path: str) -> bytes:
     with open(pdf_path, "rb") as pdf:
         pdf = pdf.read()
@@ -180,41 +147,88 @@ def get_filenames(path: str) -> list:
     return {f: guess(f).mime for f in files_grabbed if guess(f).mime}
 
 
-def structure_cardinality(
-    result: dict, single_fieldnames: dict, multi_fieldnames: dict
-):
-    """Structure the extracted invoice in different cardinalities.
+def structure_sheets(response: dict, header_sheetname="header"):
+    """Structure the extracted invoices in different sheets.
 
     Arguments:
-        result {[type]} -- [description]
-        single_fieldnames {[type]} -- [description]
-        multi_fieldnames {[type]} -- [description]
+        response {[type]} -- [description]
 
     Returns:
         [type] -- [description]
     """
-    single_cardinality = {}
+    single_cardinality = {header_sheetname: []}
     multi_cardinality = {}
 
-    for idx, document in result.items():
-        single_item = copy(single_fieldnames)
+    for idx, document in response.items():
+        single_cardinality[header_sheetname].append(
+            {
+                "file_name": document["file_name"],
+                "error_message": document.get("error_message", (None, None, None)),
+            }
+        )
         multi_items = {}
 
         for col, value in document.items():
             if col[-1].isdigit():
-                num = int(col.split("_")[-1]) + 1
-                label = "_".join(col.split("_")[:-1])
-                if num not in multi_items:
-                    multi_item = copy(multi_fieldnames)
-                    multi_item["file_name"] = document["file_name"]
-                    multi_item["row_number"] = (num, None, None)
-                    multi_items[num] = multi_item
+                item_num = int(col.split("_")[-1])
+                label = col.split("_")[1]
+                category = col.split("_")[0]
+                multi_items.setdefault(category, [])
 
-                multi_items[num][label] = document[col]
+                if len(multi_items[category]) == item_num:
+                    multi_items[category].append(
+                        {
+                            "file_name": document["file_name"],
+                            "row_number": (item_num + 1, None, None),
+                        }
+                    )
+
+                multi_items[category][item_num][label] = document[col]
             else:
-                single_item[col] = document[col]
+                single_cardinality[header_sheetname][idx][col] = document[col]
 
-        single_cardinality[idx] = single_item
-        multi_cardinality[idx] = multi_items
+        for category, rows in multi_items.items():
+            for row in rows:
+                multi_cardinality.setdefault(category, []).append(row)
 
-    return single_cardinality, multi_cardinality
+    return {**single_cardinality, **multi_cardinality}
+
+
+def get_output_path(input_path, output_path, extension="xlsx"):
+    if not output_path:
+        output_path = (
+            f"{os.path.normpath(input_path).split(os.path.sep)[-1]}.{extension}"
+        )
+    else:
+        output_path = f"{os.path.join(os.getcwd(), output_path)}"
+        if not os.path.isdir(os.path.sep.join(output_path.split(os.path.sep)[:-1])):
+            quit("Output directory does not exist")
+
+    return output_path
+
+
+def structure_sheet(records):
+    columns = set()
+
+    # Collect all column names
+    for record in records:
+        columns = columns.union(set(record.keys()))
+
+    for idx, record in enumerate(records.copy()):
+        # Insert all missing columns into record
+        record.update(
+            {k: (None, None, None) for k in columns if k not in record.keys()}
+        )
+
+        # All snake_case keys
+        record = {
+            re.sub(r"(?<!^)(?=[A-Z])", "_", k).lower(): v for k, v in record.items()
+        }
+
+        # Reorder column order
+        records[idx] = {
+            **dict(itertools.islice(record.items(), 2)),
+            **dict(sorted(itertools.islice(record.items(), 2, None))),
+        }
+
+    return records
